// Copyright 2010 William Malone (www.williammalone.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

var canvas;
var context;
var canvasWidth = window.innerWidth;
var canvasHeight = window.innerHeight;
var padding = 25;
var lineWidth = 8;
var colorPurple = "#cb3594";
var colorGreen = "#659b41";
var colorYellow = "#ffcf33";
var colorBrown = "#986928";
var outlineImage = new Image();
var crayonImage = new Image();
var markerImage = new Image();
var eraserImage = new Image();
var crayonBackgroundImage = new Image();
var markerBackgroundImage = new Image();
var eraserBackgroundImage = new Image();
var crayonTextureImage = new Image();
var clickX = new Array();
var clickY = new Array();
var clickColor = new Array();
var clickTool = new Array();
var clickSize = new Array();
var clickDrag = new Array();
var paint = true;
var curColor = "#009ee0";
var curTool = "marker";
var curSize = "normal";
var mediumStartX = 18;
var mediumStartY = 19;
var mediumImageWidth = 93;
var mediumImageHeight = 46;
var drawingAreaX = 111;
var drawingAreaY = 11;
var drawingAreaWidth = 267;
var drawingAreaHeight = 200;
var toolHotspotStartY = 23;
var toolHotspotHeight = 38;
var sizeHotspotStartY = 157;
var sizeHotspotHeight = 36;
var sizeHotspotWidthObject = new Object();
sizeHotspotWidthObject.huge = 39;
sizeHotspotWidthObject.large = 25;
sizeHotspotWidthObject.normal = 18;
sizeHotspotWidthObject.small = 16;

function executeArticleScript() {
	//console.log("executeArticleScript");
	/*prepareCanvas();
	prepareSimpleCanvas();
	prepareSimpleColorsCanvas();
	prepareSimpleSizesCanvas();*/
	prepareSimpleToolsCanvas();
	//prepareSimpleOutlineCanvas();

}

var totalLoadResources = 8
var curLoadResNum = 8;
/**
* Calls the redraw function after all neccessary resources are loaded.
*/
function resourceLoaded()
{
	if(++curLoadResNum >= totalLoadResources - 1){
		redraw();
	}
}

/**
* Creates a canvas element, loads images, adds events, and draws the canvas for the first time.
*/
function prepareCanvas()
{
	// Create the canvas (Neccessary for IE because it doesn't know what a canvas element is)
	var canvasDiv = document.getElementById('canvasDiv');
	canvas = document.createElement('canvas');
	canvas.setAttribute('width', canvasWidth);
	canvas.setAttribute('height', canvasHeight);
//	canvas.setAttribute('style', 'position:absolute;');
	canvas.style.position = "absolute";
	canvas.setAttribute('id', 'canvas');
	canvasDiv.appendChild(canvas);
	
	if(typeof G_vmlCanvasManager != 'undefined') {
		canvas = G_vmlCanvasManager.initElement(canvas);
	}
	context = canvas.getContext("2d"); // Grab the 2d canvas context
	// Note: The above code is a workaround for IE 8 and lower. Otherwise we could have used:
	//     context = document.getElementById('canvas').getContext("2d");
	
	// Load images
	// -----------
	crayonImage.onload = function() { resourceLoaded(); 
	}
	crayonImage.src = "images/crayon-outline.png";
	
	markerImage.onload = function() { resourceLoaded(); 
	}
	markerImage.src = "images/marker-outline.png";
	
	eraserImage.onload = function() { resourceLoaded(); 
	}
	eraserImage.src = "images/eraser-outline.png";	
	
	crayonBackgroundImage.onload = function() { resourceLoaded(); 
	}
	crayonBackgroundImage.src = "images/crayon-background.png";
	
	markerBackgroundImage.onload = function() { resourceLoaded(); 
	}
	markerBackgroundImage.src = "images/marker-background.png";
	
	eraserBackgroundImage.onload = function() { resourceLoaded(); 
	}
	eraserBackgroundImage.src = "images/eraser-background.png";

	crayonTextureImage.onload = function() { resourceLoaded(); 
	}
	crayonTextureImage.src = "images/crayon-texture.png";
	
	outlineImage.onload = function() { resourceLoaded(); 
	}
	outlineImage.src = "images/watermelon-duck-outline.png";
	
	// Add mouse events
	// ----------------
	$('#canvas').mousedown(function(e)
	{
		// Mouse down location
		var mouseX = e.pageX - this.offsetLeft;
		var mouseY = e.pageY - this.offsetTop;
		
		if(mouseX < drawingAreaX) // Left of the drawing area
		{
			if(mouseX > mediumStartX)
			{
				if(mouseY > mediumStartY && mouseY < mediumStartY + mediumImageHeight){
					curColor = colorPurple;
				}else if(mouseY > mediumStartY + mediumImageHeight && mouseY < mediumStartY + mediumImageHeight * 2){
					curColor = colorGreen;
				}else if(mouseY > mediumStartY + mediumImageHeight * 2 && mouseY < mediumStartY + mediumImageHeight * 3){
					curColor = colorYellow;
				}else if(mouseY > mediumStartY + mediumImageHeight * 3 && mouseY < mediumStartY + mediumImageHeight * 4){
					curColor = colorBrown;
				}
			}
		}
		else if(mouseX > drawingAreaX + drawingAreaWidth) // Right of the drawing area
		{
			if(mouseY > toolHotspotStartY)
			{
				if(mouseY > sizeHotspotStartY)
				{
					var sizeHotspotStartX = drawingAreaX + drawingAreaWidth;
					if(mouseY < sizeHotspotStartY + sizeHotspotHeight && mouseX > sizeHotspotStartX)
					{
						if(mouseX < sizeHotspotStartX + sizeHotspotWidthObject.huge){
							curSize = "huge";
						}else if(mouseX < sizeHotspotStartX + sizeHotspotWidthObject.large + sizeHotspotWidthObject.huge){
							curSize = "large";
						}else if(mouseX < sizeHotspotStartX + sizeHotspotWidthObject.normal + sizeHotspotWidthObject.large + sizeHotspotWidthObject.huge){
							curSize = "normal";
						}else if(mouseX < sizeHotspotStartX + sizeHotspotWidthObject.small + sizeHotspotWidthObject.normal + sizeHotspotWidthObject.large + sizeHotspotWidthObject.huge){
							curSize = "small";						
						}
					}
				}
				else
				{
					if(mouseY < toolHotspotStartY + toolHotspotHeight){
						curTool = "crayon";
					}else if(mouseY < toolHotspotStartY + toolHotspotHeight * 2){
						curTool = "marker";
					}else if(mouseY < toolHotspotStartY + toolHotspotHeight * 3){
						curTool = "eraser";
					}
				}
			}
		}
		else if(mouseY > drawingAreaY && mouseY < drawingAreaY + drawingAreaHeight)
		{
			// Mouse click location on drawing area
		}
		paint = true;
		addClick(mouseX, mouseY, false);
		redraw();
	});
	
	$('#canvas').mousemove(function(e){
		if(paint==true){
			addClick(e.pageX - this.offsetLeft, e.pageY - this.offsetTop, true);
			redraw();
		}
	});
	
	$('#canvas').mouseup(function(e){
		paint = false;
	  	redraw();
	});
	
	$('#canvas').mouseleave(function(e){
		paint = false;
	});
}

/**
* Adds a point to the drawing array.
* @param x
* @param y
* @param dragging
*/
function addClick(x, y, dragging)
{
	clickX.push(x);
	clickY.push(y);
	clickTool.push(curTool);
	clickColor.push(curColor);
	clickSize.push(curSize);
	clickDrag.push(dragging);
}

/**
* Clears the canvas.
*/
function clearCanvas()
{
	context.clearRect(0, 0, canvasWidth, canvasHeight);
}

/**
* Redraws the canvas.
*/
function redraw()
{
	// Make sure required resources are loaded before redrawing
	if(curLoadResNum < totalLoadResources){ return; }
	
	clearCanvas();
	
	var locX, locY;
	if(curTool == "crayon")
	{
		// Draw the crayon tool background
		context.drawImage(crayonBackgroundImage, 0, 0, canvasWidth, canvasHeight);
		
		// Purple
		locX = (curColor == colorPurple) ? 18 : 52;
		locY = 19;
		
		context.beginPath();
		context.moveTo(locX + 41, locY + 11);
		context.lineTo(locX + 41, locY + 35);
		context.lineTo(locX + 29, locY + 35);
		context.lineTo(locX + 29, locY + 33);
		context.lineTo(locX + 11, locY + 27);
		context.lineTo(locX + 11, locY + 19);
		context.lineTo(locX + 29, locY + 13);
		context.lineTo(locX + 29, locY + 11);
		context.lineTo(locX + 41, locY + 11);
		context.closePath();
		context.fillStyle = colorPurple;
		context.fill();	

		if(curColor == colorPurple){
			context.drawImage(crayonImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(crayonImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
		
		// Green
		locX = (curColor == colorGreen) ? 18 : 52;
		locY += 46
;		
		context.beginPath();
		context.moveTo(locX + 41, locY + 11);
		context.lineTo(locX + 41, locY + 35);
		context.lineTo(locX + 29, locY + 35);
		context.lineTo(locX + 29, locY + 33);
		context.lineTo(locX + 11, locY + 27);
		context.lineTo(locX + 11, locY + 19);
		context.lineTo(locX + 29, locY + 13);
		context.lineTo(locX + 29, locY + 11);
		context.lineTo(locX + 41, locY + 11);
		context.closePath();
		context.fillStyle = colorGreen;
		context.fill();	

		if(curColor == colorGreen){
			context.drawImage(crayonImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(crayonImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
		
		// Yellow
		locX = (curColor == colorYellow) ? 18 : 52;
		locY += 46;
		
		context.beginPath();
		context.moveTo(locX + 41, locY + 11);
		context.lineTo(locX + 41, locY + 35);
		context.lineTo(locX + 29, locY + 35);
		context.lineTo(locX + 29, locY + 33);
		context.lineTo(locX + 11, locY + 27);
		context.lineTo(locX + 11, locY + 19);
		context.lineTo(locX + 29, locY + 13);
		context.lineTo(locX + 29, locY + 11);
		context.lineTo(locX + 41, locY + 11);
		context.closePath();
		context.fillStyle = colorYellow;
		context.fill();	

		if(curColor == colorYellow){
			context.drawImage(crayonImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(crayonImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
		
		// Yellow
		locX = (curColor == colorBrown) ? 18 : 52;
		locY += 46;
		
		context.beginPath();
		context.moveTo(locX + 41, locY + 11);
		context.lineTo(locX + 41, locY + 35);
		context.lineTo(locX + 29, locY + 35);
		context.lineTo(locX + 29, locY + 33);
		context.lineTo(locX + 11, locY + 27);
		context.lineTo(locX + 11, locY + 19);
		context.lineTo(locX + 29, locY + 13);
		context.lineTo(locX + 29, locY + 11);
		context.lineTo(locX + 41, locY + 11);
		context.closePath();
		context.fillStyle = colorBrown;
		context.fill();	

		if(curColor == colorBrown){
			context.drawImage(crayonImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(crayonImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
	}
	else if(curTool == "marker")
	{
		// Draw the marker tool background
		context.drawImage(markerBackgroundImage, 0, 0, canvasWidth, canvasHeight);
		
		// Purple
		locX = (curColor == colorPurple) ? 18 : 52;
		locY = 19;
		
		context.beginPath();
		context.moveTo(locX + 10, locY + 24);
		context.lineTo(locX + 10, locY + 24);
		context.lineTo(locX + 22, locY + 16);
		context.lineTo(locX + 22, locY + 31);
		context.closePath();
		context.fillStyle = colorPurple;
		context.fill();	

		if(curColor == colorPurple){
			context.drawImage(markerImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(markerImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
		
		// Green
		locX = (curColor == colorGreen) ? 18 : 52;
		locY += 46;
		
		context.beginPath();
		context.moveTo(locX + 10, locY + 24);
		context.lineTo(locX + 10, locY + 24);
		context.lineTo(locX + 22, locY + 16);
		context.lineTo(locX + 22, locY + 31);
		context.closePath();
		context.fillStyle = colorGreen;
		context.fill();	

		if(curColor == colorGreen){
			context.drawImage(markerImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(markerImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
		
		// Yellow
		locX = (curColor == colorYellow) ? 18 : 52;
		locY += 46;
		
		context.beginPath();
		context.moveTo(locX + 10, locY + 24);
		context.lineTo(locX + 10, locY + 24);
		context.lineTo(locX + 22, locY + 16);
		context.lineTo(locX + 22, locY + 31);
		context.closePath();
		context.fillStyle = colorYellow;
		context.fill();	

		if(curColor == colorYellow){
			context.drawImage(markerImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(markerImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
		
		// Yellow
		locX = (curColor == colorBrown) ? 18 : 52;
		locY += 46;
		
		context.beginPath();
		context.moveTo(locX + 10, locY + 24);
		context.lineTo(locX + 10, locY + 24);
		context.lineTo(locX + 22, locY + 16);
		context.lineTo(locX + 22, locY + 31);
		context.closePath();
		context.fillStyle = colorBrown;
		context.fill();	

		if(curColor == colorBrown){
			context.drawImage(markerImage, locX, locY, mediumImageWidth, mediumImageHeight);
		}else{
			context.drawImage(markerImage, 0, 0, 59, mediumImageHeight, locX, locY, 59, mediumImageHeight);
		}
	}
	else if(curTool == "eraser")
	{
		context.drawImage(eraserBackgroundImage, 0, 0, canvasWidth, canvasHeight);
		context.drawImage(eraserImage, 18, 19, mediumImageWidth, mediumImageHeight);	
	}
	
	if(curSize == "small"){
		locX = 467;
	}else if(curSize == "normal"){
		locX = 450;
	}else if(curSize == "large"){
		locX = 428;
	}else if(curSize == "huge"){
		locX = 399;
	}
	locY = 189;
	context.beginPath();
	context.rect(locX, locY, 2, 12);
	context.closePath();
	context.fillStyle = '#333333';
	context.fill();	
	
	// Keep the drawing in the drawing area
	context.save();
	context.beginPath();
	context.rect(drawingAreaX, drawingAreaY, drawingAreaWidth, drawingAreaHeight);
	context.clip();
		
	var radius;
	var i = 0;
	for(; i < clickX.length; i++)
	{		
		if(clickSize[i] == "small"){
			radius = 2;
		}else if(clickSize[i] == "normal"){
			radius = 5;
		}else if(clickSize[i] == "large"){
			radius = 10;
		}else if(clickSize[i] == "huge"){
			radius = 20;
		}else{
			radius = 0;	
		}
		
		context.beginPath();
		if(clickDrag[i] && i){
			context.moveTo(clickX[i-1], clickY[i-1]);
		}else{
			context.moveTo(clickX[i], clickY[i]);
		}
		context.lineTo(clickX[i], clickY[i]);
		context.closePath();
		
		if(clickTool[i] == "eraser"){
			/*context.globalCompositeOperation = "destination-out";
			context.strokeStyle = 'rgba(255,255,255,1.0)';
			context.beginPath();
			context.arc(clickX[i], clickY[i], radius, 0, 2 * Math.PI, false);
			context.clip();
			context.clearRect(xclickX[i] - radius - 1, clickY[i] - radius - 1,
				radius * 2 + 2, radius * 2 + 2);*/
			context.strokeStyle = "white";
		}else{

			//context.globalCompositeOperation = "source-over";	
			context.strokeStyle = clickColor[i];
	
		}
		context.lineJoin = "round";
		context.lineWidth = radius;
		context.stroke();		
		
	}
	//context.globalCompositeOperation = "source-over";
	context.restore();
	
	// Overlay a crayon texture (if the current tool is crayon)
	if(curTool == "crayon"){
		context.globalAlpha = 0.4; // No IE support
		context.drawImage(crayonTextureImage, 0, 0, canvasWidth, canvasHeight);
	}
	context.globalAlpha = 1; // No IE support
	
	// Draw the outline image
	context.drawImage(outlineImage, drawingAreaX, drawingAreaY, drawingAreaWidth, drawingAreaHeight);
}

/****************************************************************************** Simple Canvas With Tools */

var clickX_simpleTools = new Array();
var clickY_simpleTools = new Array();
var clickDrag_simpleTools = new Array();
var clickColor_simpleTools = new Array();
var clickSize_simpleTools = new Array();
var paint_simpleTools;
var canvas_simpleTools;
var context_simpleTools;
var curColor_simpleTools = "#009ee0";
var curSize_simpleTools = "normal";

var clickTool_simpleTools = new Array();
var curTool_simpleTools = "crayon";

function activarCanvas() {
	if($("#canvasSimpleTools").length == 0){
		executeArticleScript();
	}
    canvas_simpleTools.setAttribute('style', "left:0;top:0;bottom:0;right:0;position:absolute;z-index:9999999;pointer-events:auto;");
}

function desactivarCanvas() {
    canvas_simpleTools.setAttribute('style', "left:0;top:0;bottom:0;right:0;position:absolute;z-index:9999999;pointer-events:none;");
}

function grosorCanvas(ngrosor) {
    if (ngrosor === 1) {
        curSize_simpleTools = "small";
    } else if (ngrosor === 2) {
        curSize_simpleTools = "normal";
    } else if (ngrosor === 3) {
        curSize_simpleTools = "large";
    } else if (ngrosor === 4) {
        curSize_simpleTools = "huge";
    }
}

function colorCanvas(scolor) {
    curColor_simpleTools = scolor;
}

function tipoCanvas(stipo) {
    curTool_simpleTools = stipo;
}


function prepareSimpleToolsCanvas()
{	
	// Create the canvas (Neccessary for IE because it doesn't know what a canvas element is)
	var canvasDiv = document.getElementById('canvasSimpleToolsDiv');
	canvas_simpleTools = document.createElement('canvas');
	canvas_simpleTools.setAttribute('width', canvasWidth);
	canvas_simpleTools.setAttribute('height', canvasHeight);
	canvas_simpleTools.setAttribute('style', "left:0;top:0;bottom:0;right:0;position:absolute;z-index:999;pointer-events:none;");
	canvas_simpleTools.setAttribute('id', 'canvasSimpleTools');
	canvasDiv.appendChild(canvas_simpleTools);
	if(typeof G_vmlCanvasManager != 'undefined') {
		canvas_simpleTools = G_vmlCanvasManager.initElement(canvas_simpleTools);
	}

	context_simpleTools = canvas_simpleTools.getContext("2d"); // Grab the 2d canvas context
	// Note: The above code is a workaround for IE 8 and lower. Otherwise we could have used:
	//     context = document.getElementById('canvas').getContext("2d");
	
	// Add mouse events
	// ----------------
	$('#canvasSimpleTools').mousedown(function(e)
	{
		// Mouse down location
		var mouseX = e.pageX - this.offsetLeft;
		var mouseY = e.pageY - this.offsetTop;
		paint_simpleTools = true;
		addClickSimpleTools(mouseX, mouseY, false);
		redrawSimpleTools();
	});
	
	$('#canvasSimpleTools').mousemove(function(e){
		if(paint_simpleTools){
			addClickSimpleTools(e.pageX - this.offsetLeft, e.pageY - this.offsetTop, true);
			redrawSimpleTools();
		}
	});
	
	$('#canvasSimpleTools').mouseup(function(e){
		paint_simpleTools = false;
	  	redrawSimpleTools();
	});
	
	$('#canvasSimpleTools').mouseleave(function(e){
		paint_simpleTools = false;
	});

	// Add touch event listeners to canvas element
	canvas_simpleTools.addEventListener("touchstart", function(e){
		// Mouse down location
		var mouseX = (e.changedTouches ? e.changedTouches[0].pageX : e.pageX) - this.offsetLeft,
			mouseY = (e.changedTouches ? e.changedTouches[0].pageY : e.pageY) - this.offsetTop;
		
		paint_simpleTools = true;
		addClickSimpleTools(mouseX, mouseY, false);
		redrawSimpleTools();
	}, false);
	
	canvas_simpleTools.addEventListener("touchmove", function(e){
		var mouseX = (e.changedTouches ? e.changedTouches[0].pageX : e.pageX) - this.offsetLeft,
			mouseY = (e.changedTouches ? e.changedTouches[0].pageY : e.pageY) - this.offsetTop;
					
		if(paint_simpleTools){
			addClickSimpleTools(mouseX, mouseY, true);
			addClickSimpleTools(mouseX, mouseY, false);
			redrawSimpleTools();
		}
		e.preventDefault();
	}, false);

	canvas_simpleTools.addEventListener("touchend", function(e){
		paint_simpleTools = false;
	  	redrawSimpleTools();
	}, false);

	canvas_simpleTools.addEventListener("touchcancel", function(e){
		paint_simpleTools = false;
	}, false);

	$('#choosePurpleSimpleTools').mousedown(function(e){
		curColor_simpleTools = colorPurple;
	});
	$('#chooseGreenSimpleTools').mousedown(function(e){
		curColor_simpleTools = colorGreen;
	});
	$('#chooseYellowSimpleTools').mousedown(function(e){
		curColor_simpleTools = colorYellow;
	});
	$('#chooseBrownSimpleTools').mousedown(function(e){
		curColor_simpleTools = colorBrown;
	});	
	$('#chooseSmallSimpleTools').mousedown(function(e){
		curSize_simpleTools = "small";
	});
	$('#chooseNormalSimpleTools').mousedown(function(e){
		curSize_simpleTools = "normal";
	});
	$('#chooseLargeSimpleTools').mousedown(function(e){
		curSize_simpleTools = "large";
	});
	$('#chooseHugeSimpleTools').mousedown(function(e){
	    curSize_simpleTools = "huge";

	});
	$('#chooseCrayonSimpleTools').mousedown(function(e){
		curTool_simpleTools = "crayon";
	});
	$('#chooseMarkerSimpleTools').mousedown(function(e){
		curTool_simpleTools = "marker";
	});
	$('#chooseEraserSimpleTools').mousedown(function(e){
		curTool_simpleTools = "eraser";
	});
	
	$('#clearCanvasSimpleTools').mousedown(function(e)
	{
		clickX_simpleTools = new Array();
		clickY_simpleTools = new Array();
		clickDrag_simpleTools = new Array();
		clickColor_simpleTools = new Array();
		clickSize_simpleTools = new Array();
		clearCanvas_simpleTools();
	});
}

function addClickSimpleTools(x, y, dragging)
{
	clickX_simpleTools.push(x);
	clickY_simpleTools.push(y);
	clickDrag_simpleTools.push(dragging);
	clickTool_simpleTools.push(curTool_simpleTools);
	clickColor_simpleTools.push(curColor_simpleTools);
	clickSize_simpleTools.push(curSize_simpleTools);
}

function clearCanvas_simpleTools()
{
	context_simpleTools.clearRect(0, 0, canvasWidth, canvasHeight);
}
var i=0;
function redrawSimpleTools()
{
	// Make sure required resources are loaded before redrawing

	if(curLoadResNum < totalLoadResources){ return; }
			
	//clearCanvas_simpleTools();
	
	var radius;
	context_simpleTools.lineJoin = "round";
			
	for(i=i; i < clickX_simpleTools.length; i++)
	{
		if(clickSize_simpleTools[i] == "small"){
			radius = 2;
		}else if(clickSize_simpleTools[i] == "normal"){
			radius = 5;
		}else if(clickSize_simpleTools[i] == "large"){
			radius = 10;
		}else if(clickSize_simpleTools[i] == "huge"){
			radius = 20;
		}

		if(curTool_simpleTools == "eraser"){
			context_simpleTools.globalCompositeOperation = "destination-out";
		}else{
			context_simpleTools.globalCompositeOperation = "source-over";
		}

		context_simpleTools.beginPath();
		if(clickDrag_simpleTools[i] && i){
			context_simpleTools.moveTo(clickX_simpleTools[i-1], clickY_simpleTools[i-1]);
		}else{
			context_simpleTools.moveTo(clickX_simpleTools[i]-1, clickY_simpleTools[i]);
		}
		context_simpleTools.lineTo(clickX_simpleTools[i], clickY_simpleTools[i]);
		context_simpleTools.closePath();
		context_simpleTools.strokeStyle = clickColor_simpleTools[i];
		context_simpleTools.lineWidth = radius;
		context_simpleTools.stroke();

	}
	
	if(curTool_simpleTools == "crayon"){
		context_simpleTools.drawImage(crayonTextureImage, 0, 0, canvasWidth, canvasHeight);
	}
}

$('*[href="#"]').attr("href","javascript:void(0);");

